// ===================================================================
// (C) Copyright IBM Corp. 2005, 2006 All Rights Reserved.
//                                                                           
// Licensed Materials - Property of IBM
//                                                          
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//              
// You may copy, modify, and distribute this sample program in any
// form without payment to IBM, for any purpose including developing,
// using, marketing or distributing programs that include or are
// derivative works of the sample program.
//                                                                           
// The sample program is provided to you on an "AS IS" basis, without
// warranty of any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES,
// EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Some jurisdictions do not allow for the exclusion or limitation of
// implied warranties, so the above limitations or exclusions may not
// apply to you. IBM shall not be liable for any damages you suffer as
// a result of using, modifying or distributing the sample program or
// its derivatives.
//===================================================================
package com.ibm.wbimonitor.samples.fileemitter;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.wbimonitor.samples.fileemitter.exception.EventFolderPollerException;

/*
 * Polls the inboundFolder for inbound event files and archives(purges) these
 * event files after processing.
 */
public class EventFolderPoller {

    private static Logger logger = Logger.getLogger( "EventFolderPoller" );

    private static final int POLL_SIZE_DEFAULT = 50;

    private static File inboundFolder;

    private static File faultFolder;

    private static File archiveFolder;

    /*
     * Static initialization of the inbound, archive and fault folders.
     */
    static {
        try {
            InitialContext ic = new InitialContext( );
            inboundFolder = new File( (String)ic.lookup( "java:comp/env/inboundEventsDirectory" ) );
            faultFolder = new File( (String)ic.lookup( "java:comp/env/faultDirectory" ) );
            archiveFolder = new File( (String)ic.lookup( "java:comp/env/archivedEventsDirectory" ) );
        }
        catch( NamingException ne ) {
            logger.log( Level.INFO, "Error encountered while trying to determine the values for the inbouund, archive and fault folders." );
            logger.log( Level.INFO, "[inboundFolder = " + inboundFolder + "; archiveFolder = " + archiveFolder + "; faultFolder = " + faultFolder + "]" );
            logger.log( Level.INFO, ne.toString( ) );
        }
    }

    /*
     * Poll the inboundFolder for inbound events/files. Up to POLL_SIZE_DEFAULT
     * (POLL_SIZE_DEFAULT = 50) will be processed per polling period. The
     * returned collection of EventFileBeans is sorted based on the last
     * modified time (using the compareTo implementation of the Comparable
     * interface provided in the EventFileBean class)
     */
    public static Collection poll( ) throws EventFolderPollerException {
        ArrayList eventFileBeansList = new ArrayList( );

        if( ( inboundFolder == null ) ) {
            throw new EventFolderPollerException( "The inbound events folder is not specified.  Ensure it has been specified in the deployment descriptor environment property." );
        }

        if( !( inboundFolder.exists( ) ) ) {
            throw new EventFolderPollerException( "The inbound events folder, " + inboundFolder + ", does not exist." );
        }

        File[] inboundEvents = inboundFolder.listFiles( );
        for( int i = 0; ( ( i < inboundEvents.length ) && ( i <= POLL_SIZE_DEFAULT ) ); i++ ) {
            File inboundEvent = inboundEvents[i];
            EventFileBean efb = new EventFileBean( );
            getFileContents( efb, inboundEvent );
            eventFileBeansList.add( efb );
        }
        Collections.sort( eventFileBeansList );
        return eventFileBeansList;
    }

    /*
     * Archive the event file to the appropriate folder. If the boolean argument
     * is true (indicating successful emission of the CBE event), the event file
     * is archived to the archiveFolder. Otherwise, if it is false, the event
     * file is archived to the faultFolder.
     */
    public static void purge( EventFileBean bean, boolean failed ) throws EventFolderPollerException {
        String inboundEventFileAsString = bean.getCanonicalPath( );
        File inboundEventFile = new File( inboundEventFileAsString );
        File moveToFile;
        String moveToFileAsString = null;

        try {
            if( failed ) {
                if( ( faultFolder == null ) ) {
                    throw new EventFolderPollerException( "The fault events folder is not specified.  Ensure it has been specified in the deployment descriptor environment property." );
                }

                if( !( faultFolder.exists( ) ) ) {
                    throw new EventFolderPollerException( "The fault events folder, " + faultFolder + ", does not exist." );
                }

                moveToFile = new File( faultFolder.getCanonicalPath( ) + File.separator + inboundEventFile.getName( ) );
            }
            else {
                if( ( archiveFolder == null ) ) {
                    throw new EventFolderPollerException( "The archive events folder is not specified.  Ensure it has been specified in the deployment descriptor environment property." );
                }

                if( !( archiveFolder.exists( ) ) ) {
                    throw new EventFolderPollerException( "The archive events folder, " + archiveFolder + ", does not exist." );
                }

                moveToFile = new File( archiveFolder.getCanonicalPath( ) + File.separator + inboundEventFile.getName( ) );
            }

            moveToFileAsString = moveToFile.getCanonicalPath( );

            while( moveToFile.exists( ) ) {
                int i = moveToFileAsString.lastIndexOf( '.' );
                if( i < 0 )
                    i = moveToFileAsString.length( );
                String newMoveToFileAsString = moveToFileAsString.substring( 0, i ) + "_" + new Date( ).getTime( ) + moveToFileAsString.substring( i, moveToFileAsString.length( ) );
                moveToFile = new File( newMoveToFileAsString );
            }
        }
        catch( IOException ioe ) {
            logger.log( Level.INFO, "Unable to move " + inboundEventFileAsString + "to " + moveToFileAsString );
            throw new EventFolderPollerException( "Error occurred during archiving of the inbound event file. Could not get the destination path.", ioe );
        }

        FileMove( inboundEventFile, moveToFile );
    }

    /*
     * Collect the event file information (path and last modified time) as well
     * as the file contents into the EventFileBean.
     */
    private static void getFileContents( EventFileBean efb, File inboundEvent ) throws EventFolderPollerException {

        String inboundEventCanonicalPath;
        try {
            inboundEventCanonicalPath = inboundEvent.getCanonicalPath( );
            efb.setCanonicalPath( inboundEventCanonicalPath );
        }
        catch( IOException ioe ) {
            throw new EventFolderPollerException( "Unable to obtain canonical path for the inbound event file, " + inboundEvent + " for reading file contents.", ioe );
        }

        efb.setLastModifiedDate( inboundEvent.lastModified( ) );

        Properties properties = new Properties( );
        InputStream is = null;
        try {
            is = new FileInputStream( inboundEvent );
            properties.load( is );
        }
        catch( FileNotFoundException fnfe ) {
            throw new EventFolderPollerException( "Could not locate the specified inbound event file, " + inboundEvent + ", for reading file contents.", fnfe );
        }
        catch( IOException ioe ) {
            throw new EventFolderPollerException( "Unable to load the inbound event contents while reading file contents.", ioe );
        }
        finally {
            try {
                is.close( );
            }
            catch( IOException ioe ) {
                throw new EventFolderPollerException( "Error encountered while closing the file: " + inboundEventCanonicalPath + ".", ioe );
            }
        }

        efb.setFileContents( properties );
    }

    /*
     * Move/Rename a source file from to the specified destination.
     */
    private static void FileMove( File source, File destination ) throws EventFolderPollerException {
        boolean success = source.renameTo( destination );
        if( !success ) {
            logger.log( Level.INFO, "(!success)" );
            throw new EventFolderPollerException( "Unable to move the inbound event file, " + source.toString( ) + " to " + destination.toString( ) );
        }
    }
}